window.creator = function(elems, onComplete, onFail)
{

    const $ = require("ko/dom").window(window);
    const prefs = require("ko/prefs");
    const log = require('ko/logging').getLogger("projectCreator");
    const koFile = require("ko/file");
    const mainWindow = require("ko/windows").getMain();
    const ko = mainWindow.ko;

    var actions = {};
    var errorOccurred = false;
    var sccComponents = ko.scc.getAvailableSCCComponents();
    var sccComponent;
    var progress = require("ko/progress").get();
    var ignoreProgressMessages = false;
    var project;

    var init = () =>
    {
        $(elems.$wizard.element().getButton("finish")).attr({disabled: true, label: "Open Project"});
        $(elems.$wizard.element().getButton("back")).attr("disabled", "true");
        $(elems.$wizard.element().getButton("cancel")).attr("disabled", "true");
        elems.error.hide();

        progress.$element.attr("flex", 1);
        elems.output.$element.after(progress.element);
        elems.output.$element.after(require("ko/ui/spacer").create().element);

        progress.on("message", (message) =>
        {
            if (ignoreProgressMessages)
                return;
            
            if (message == "Done") // redundant
                return;

            printOutput(message, true, true);
        });

        var _actions = ['createDir'];
        var scc = elems.SCC.value();

        if (scc)
        {
            for (let i=0; i < sccComponents.length; i++)
            {
                if (scc == sccComponents[i].name)
                {
                    sccComponent = sccComponents[i].component;
                    break;
                }
            }

            if ( ! sccComponent)
            {
                printError("Source Code Control component could not be accessed");
                finalize();
                return;
            }
        }

        var initNewRepo = elems.newRepo.checked();
        if (scc && initNewRepo)
        {
            _actions.push('initRepo');
        }

        if (scc &&  ! elems.repoUrl.disabled() && elems.repoUrl.value().trim())
        {
            _actions.push('checkout');
        }

        if ( ! elems.bootstrap.disabled() && elems.bootstrap.value().trim())
        {
            _actions.push('useTemplate');
        }

        if ( ! elems.runCommand.disabled() && elems.runCommand.value().trim())
        {
            _actions.push('runCommand');
        }

        _actions.push('createProject');

        runActions(_actions);
    };

    var runActions = (_actions) =>
    {
        if (errorOccurred)
        {
            finalize();
            onFail();
            return;
        }

        if ( ! _actions.length)
        {
            finalize();
            onComplete(project);
            return;
        }

        var action = _actions.shift();

        actions[action](() => runActions(_actions));
    };

    var printStep = (data, appendLineEnding=true) =>
    {
        ignoreProgressMessages = true;
        progress.message(data);
        ignoreProgressMessages = false;
        printOutput(data, appendLineEnding, false);
    };

    var printOutput = (data, appendLineEnding=true, indent=false) =>
    {
        var elem = elems.output.element;
        var isAtBottom = elem.inputField.scrollTop == elem.inputField.scrollTopMax;

        data = data.trim();
        if (indent)
        {
            data = " .. " + data.replace(/\n/g, "\n .. ");
        }

        var lineEnd = " ";
        if (appendLineEnding)
            lineEnd = "\n";

        elems.output.value(elems.output.value() + data + lineEnd);

        if (isAtBottom)
            elem.inputField.scrollTop = elem.inputField.scrollTopMax;
    };

    var printError = (data) =>
    {
        log.error("printError: " + data);

        elems.error.value(`Error occurred: ${data}`);
        elems.error.show();

        errorOccurred = true;
    };

    var finalize = () =>
    {
        if (errorOccurred)
        {
            $(elems.$wizard.element().getButton("back")).removeAttr("disabled");
            $(elems.$wizard.element().getButton("cancel")).removeAttr("disabled", "true");
        }
        else
        {
            $(elems.$wizard.element().getButton("finish")).removeAttr("disabled", "true");
        }
        

        printStep("");
        printStep("All Done!");

        progress.close();
    };

    actions.createDir = (callback) =>
    {
        var sccPath = elems.dir.value().trim();
        
        if (koFile.isDir(sccPath))
        {
            callback();
            return;
        }
        
        printStep(`Creating directory ${sccPath} .. `, false);
        koFile.mkpath(sccPath.trim());
        printOutput(`Done`);

        callback();
        return;
    };

    actions.initRepo = (callback) =>
    {
        printStep("Initializing Repository ..");

        var scc = elems.SCC.value();
        var sccPath = elems.dir.value();
        var executable;

        if (scc == "git")
            executable = prefs.getString('gitExecutable', 'git') || 'git';
        else if (scc == "hg")
            executable = prefs.getString('hgExecutable', 'hg') || 'hg';
        else
            log.error("Invalid SCC selected for initRepo: " + scc);

        var opts = {
            env: {
                GIT_TERMINAL_PROMPT: 0,
            },
            cwd: sccPath
        };

        var args = ["init"];
        if (scc == "hg")
            args.push("-y");

        require("ko/shell").run(executable, args, opts)
            .on("stdout", (data) => printOutput(data, true, true))
            .on("stderr", (data) => printOutput(data, true, true))
            .on("error", (e) => printError(e.message))
            .on("complete", () =>
            {
                printOutput("Done", true, true);
                callback();
            });
    };

    actions.checkout = (callback) =>
    {
        var sccURL = elems.repoUrl.value();
        var targetPath = elems.dir.value();

        printStep(`Checking out ${sccURL} to ${targetPath}..`);

        var asyncCallback = { "callback": (code, data) =>
        {
            if (code !== 0)
            {
                printError(`Checkout failed with code ${code} ${data}`);
            }
            else
            {
                printOutput(data, true, true);
            }

            printOutput("Done", true, true);
            callback();
        } };
        var asyncSvc = Components.classes['@activestate.com/koAsyncService;1'].
                        getService(Components.interfaces.koIAsyncService);

        try
        {
            sccComponent.checkout(sccURL, targetPath, [], asyncCallback, null);
        }
        catch (e)
        {
            printError(e.message);
            log.exception(e, "Checkout failed");
        }
    };
    
    actions.useTemplate = (callback) =>
    {
        var toolId = elems.bootstrap.value();
        var tool = ko.toolbox2.manager.toolsMgr.getToolById(toolId);
        var targetPath = elems.dir.value();
        
        printStep(`Using template: ${tool.name}`);
        
        var cb = () =>
        {
            printOutput("Done", true, true);
            progress.off("close", cb);
            callback();
            
            return false; // don't close the progress yet
        };

        progress.on("close", cb);
        
        ko.projects.useFolderTemplate(tool, targetPath, false, progress);
    };

    actions.runCommand = (callback) =>
    {
        var command = elems.runCommand.value();
        var targetPath = elems.dir.value();

        printStep(`Running Command "${command}" ..`);
        var shell = require("ko/shell");
        var newEnv = shell.getEnv();
        newEnv.GIT_TERMINAL_PROMPT = 0;
        var opts = {
            env: newEnv,
            cwd: targetPath
        };

        shell.exec(command, opts)
            .on("stdout", (data) => printOutput(data, true, true))
            .on("stderr", (data) => printOutput(data, true, true))
            .on("error", (e) => printError(e.message))
            .on("complete", () =>
            {
                printOutput("Done", true, true);
                callback();
            });
    };

    actions.createProject = (callback) =>
    {
        printStep(`Creating Project file ..`, false);

        var path = elems.dir.value();
        var name = elems.name.value() + ".komodoproject";

        var projectURI = ko.uriparse.pathToURI(koFile.join(path, name));
        project = ko.projects.manager.newProject(projectURI, false);

        if (elems.language.value())
            project.prefset.setString("projectLanguage", elems.language.value());

        if (elems.type.value())
            project.prefset.setString("projectFramework", elems.type.value());

        if (sccComponent)
            project.prefset.setString("projectSccComponent", sccComponent.name);

        printOutput("Done", true);
        callback();
    };

    init();

};